;; -*- geiser-scheme-implementation: guile -*-

;; util.scm - This file is part of Guile TD
;; Copyright (C) 2017  Christopher Howard

;; Guile TD is free software: you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; Guile TD is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

(define-module (td util)
  :export (generic-handler
           generic-unwind
           generic-catch
           get-uid
           nil-thunk
           id
           repeat))

(define generic-handler
  (lambda (key . args)
    (simple-format #t "exception key: ~S\n" key)
    (display "keys: ")
    (for-each
     (lambda (a) (simple-format #t "~S\n" a)) args)))

(define generic-unwind
  (lambda (key . args)
    (backtrace)))

(define (generic-catch proc)
         (catch #t proc generic-handler generic-unwind))

(define uid-closure
  (lambda ()
    (let ((uid -1))
      (lambda ()
        (set! uid (1+ uid))
        uid))))

(define get-uid
  (uid-closure))

(define (nil-thunk) #nil)

(define (id x) x)

(define-syntax repeat
  (syntax-rules ()
    ((_ n exp exp* ...)
     (let loop ((cnt n))
       (unless (<= cnt 0)
         exp exp* ...
         (loop (1- cnt)))))))
